;Some cool string modification functions
;Author: Shantanu Gadgil
;- Enjoy!!!

.386
.model flat, stdcall  ; 32 bit memory model
option casemap :none  ; case sensitive

include windows.inc

include kernel32.inc

include String.inc	;for EQUs

.code

;String_EndsWith
;check if a string ends with a certain other string
;pszMain: Main string _in which_ to check
;pszCheck: the string which you are looking for
;returns TRUE or FALSE
String_EndsWith	proc	pszMain:LPCTSTR, pszCheck:LPCTSTR
	push	esi
	push	edi
	
	invoke	lstrlen,pszMain
	mov	ebx,eax
	
	invoke	lstrlen,pszCheck
	;check for invalid lengths
	.if	eax > ebx
		mov	eax, FALSE
		ret
	.endif
	
	mov	esi,pszMain
	mov	edi,pszCheck
	
	.while	byte ptr [esi] != 0
		inc	esi
	.endw
	
	;reach the "beginning" of comparison start point
	.while	byte ptr [edi] != 0
		inc	edi
		dec	esi
	.endw
	
	cld
	;reset edi
	mov	edi,pszCheck
	.while	byte ptr [edi] != 0
		cmpsb
		.break	.if	!zero?
	.endw
	
	.if	zero?	;ZF is still clear
		mov	eax,TRUE
	.else
		mov	eax,FALSE
	.endif
	
	pop	edi
	pop	esi	
	
	ret
String_EndsWith endp

;String_StringInString
;search for a substring in another sting (for want of an "lstrstr" :) :)
;pszMain: Main string _in which_ you are searching
;pszChild: the substring that you are searching in the MAIN string
;bCase: case sensitive or case-insensitive search
;returns
;0 when NOT FOUND
;else pointer to the first occurence of the seacrh string FROM the MAIN string
;e.g.
;	pszMain = 'aabbcc',0
;	pszChild = 'bb',0
;	search will return pointer to 'bbcc',0 (in the main string :) )
String_StringInString	proc pszMain:LPCTSTR, pszChild:LPCTSTR, bCase:BOOL
	push	esi
	push	edi
	push	ebx
	push	edx
	
	mov	esi, pszMain
	mov	ebx, pszMain
	mov	edi, pszChild
	
	.while	byte ptr [edi] != 0
		.if	byte ptr [esi] == 0
			.break
		.endif
		
		mov	al, [esi]
		mov	dl, [edi]
		
		.if	bCase == FALSE	;NOT case sensitive, make smaller case
			.if	al >= 'A' && al <= 'Z'
				or	al, 32
			.endif
			
			.if	dl >= 'A' && dl <= 'Z'
				or	dl, 32
			.endif
		.endif
		
		.if	al == dl
			inc	esi
			inc	edi
		.else
			inc	esi
			mov	ebx, esi	;last occurence where they were NOT equal
			mov	edi, pszChild
		.endif
	.endw
	
	.if	byte ptr [edi] == 0	;edi reached 0 means scan was successful :)
		mov	eax,ebx		;return pointer to first occurence of the substring
	.else
		xor	eax,eax		;return NULL :)
	.endif
	
	pop	edx
	pop	ebx
	pop	edi
	pop	esi
	ret
String_StringInString	endp

;String_CommaFormat
;usually for number strings to comma separate it into groups of three
;e.g:
;	pszSource = 123456789
;	pszTarget = 123,456,789
;be careful...no checks for overlap, etc!!!
String_CommaFormat	proc	pszSource:LPCTSTR, pszTarget:LPTSTR
	push	eax
	push	ecx
	
	push	esi
	push	edi
	
	mov	esi,pszSource
	mov	edi,pszTarget
	
	mov	ecx,0ffffffffh
	.while	byte ptr [esi] != 0
		inc	esi
		inc	edi
		
		inc	ecx
		.if	ecx == 3
			inc	edi		;space for commas
			xor	ecx,ecx
		.endif
	.endw
	
	mov	ecx,0ffffffffh
	.while	esi >= pszSource
		mov	al,[esi]
		mov	[edi],al
		
		dec	esi
		dec	edi
		
		inc	ecx
		.if	ecx == 3
			mov	byte ptr [edi],','
			dec	edi
			
			xor	ecx,ecx
		.endif
	.endw
	
	pop	edi
	pop	esi
	
	pop	ecx
	pop	eax

	ret
String_CommaFormat endp

;String_ChangeCase
;formats the given string _in place_ with the specified options
;pszString	= the string to change
;iCase		= convert to what case; STRING_CHANGE_CASE_UPPER, STRING_CHANGE_CASE_LOWER, STRING_CHANGE_CASE_PROPER
;pszDelimiters	= list of delimiters (for "Proper Case" only)
;pszIgnoreChars	= list of characters (in pairs) within which to ignore the modification (for "Proper Case" only)
;	e.g:
;		pszDelimiters = ' .',0	;SPACE and DOT
;		pszIgnoreChars = '{}[]',0
;		pszString = 'tvserial name.episode number.{SOME TEXT}.[ENCODER DUDE].avi',0
;		ProperCase will make it into 'Tvserial Name.Episode Number.{SOME TEXT}.[ENCODER DUDE].avi',0
;
;		the stuff after the last '.' is assumed to be the file extension and is CONVERTED to small case :)
String_ChangeCase	proc	pszString:LPTSTR, iCase:SDWORD, pszDelimiters:LPCTSTR, pszIgnoreChars:LPCTSTR
	LOCAL	bNewWord:BOOL
	
	push	ebx
	push	ecx
	push	esi
	push	edi
	
	xor	ecx,ecx
	mov	esi,pszIgnoreChars
	.while	byte ptr [esi] != 0
		inc	ecx
		inc	esi
	.endw
	and	ecx,1		;check if count is ODD
	.if	ecx == 1	;it is ODD :(
		mov	eax,FALSE
		ret
	.endif
	mov	bNewWord,TRUE
	
	mov	esi,pszString
	.while	byte ptr [esi] != 0
		;lets check for ignore in-between charaters
		mov	ebx,pszIgnoreChars
		.while	byte ptr [ebx] != 0
			mov	al,[esi]
			sub	al,[ebx]
			
			.if	al == 0		;first in the IGNORE PAIR found
				inc	esi
				inc	ebx	;pointing to the SECOND char in the IGNORE PAIR
				.while	byte ptr [esi] != 0	;run ONLY till end of string :)
					mov	al,[esi]
					sub	al,[ebx]
					inc	esi
					.if	al == 0
						.break
					.endif
				.endw
				.break
			.endif
			inc	ebx
		.endw
		
		.if	byte ptr [esi] == 0
			;can happen if only the FIRST of the
			;IGNORE-PAIR exists in the string :(
			.break
		.endif
		
		.if	iCase == STRING_CHANGE_CASE_LOWER || iCase == STRING_CHANGE_CASE_PROPER
			;lets make the character into small case
			mov	al,[esi]
			.if	al >= 'A' && al <= 'Z'
				add	al,32
				mov	[esi],al
			.endif
		.elseif	iCase == STRING_CHANGE_CASE_UPPER
			mov	al,[esi]
			.if	al >= 'a' && al <= 'z'
				sub	al,32
				mov	[esi],al
			.endif
		.endif
		
		.if	iCase != STRING_CHANGE_CASE_PROPER
			inc	esi
			.continue
		.endif
		
		;if it is a new word, CAPITALIZE it ! :)
		.if	bNewWord == TRUE
			mov	al,[esi]
			.if	al >= 'a' && al <= 'z'
				sub	al,32
				mov	[esi],al
			.endif
			
			mov	bNewWord,FALSE
		.endif
		
		;lets check for delimiter character and skip
		;ALL consecutive delimiter characters
		mov	edi,pszDelimiters
		.while	byte ptr [edi] != 0
			mov	al,[esi]
			sub	al,[edi]
			.if	al == 0		;delimiter char found !
				inc	esi
				mov	bNewWord,TRUE
				mov	edi,pszDelimiters
			.else
				inc	edi
			.endif
			
			.break	.if	byte ptr [esi] == 0
		.endw
		
		.break	.if	byte ptr [esi] == 0
		
		.if	bNewWord == FALSE
			inc	esi
		.endif
	.endw
	
	.if	iCase != STRING_CHANGE_CASE_PROPER
		jmp	@f
	.endif
	;we have reached end of string
	;check if there exists a '.' character in the file name
	;if it exists, keep the extension of the file in LOWER CASE :)
	mov	ebx,esi
	.while	ebx > pszString
		.if	byte ptr [ebx] == '.'
			.break
		.endif
		dec	ebx
	.endw
	
	.if	ebx > pszString
		dec	esi
		.while	1
			.if	esi == ebx
				.break
			.endif
			
			mov	al,[esi]
			.if	al >= 'A' && al <= 'Z'
				add	al,32
				mov	[esi],al
			.endif
			dec	esi
		.endw
	.endif
@@:
	mov	eax,TRUE
	
	pop	edi
	pop	esi
	pop	ecx
	pop	ebx	
	ret

String_ChangeCase endp

end

